<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Clase para manejar la conexion de base de datos
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */

class Database extends Singleton
{
	//**********************************************************************************
	// Constantes
	//**********************************************************************************
 
	/**
	 * Instancia única de la clase
	 * @var Database
	 */
	
	private static $instance;
	
	/**
	 * Los resultados se muestran en un objeto
	 * @var int
	 */

	const F_OBJECT = 101;
	
	/**
	 * Los resultados se muestran en una matriz numérica
	 * @var int
	 */
	
	const F_ROW = 102;

	/**
	 * Los resultados se muestran en una matriz asociativa
	 * @var int
	 */
	 
	const F_ASSOC = 103;


	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************

	/**
	 * Dominio de la base de datos
	 * @var string
	 */
	 
	private $host =  "localhost";

	/**
	 * Nombre de usuario 
	 * @var string
	 */
	 
	private $user = "root";

	/**
	 * Contraseña de usuario
	 * @var string
	 */
	 
	private $password = "";

	/**
	 * Nombre de la base de datos
	 * @var string
	 */
	 
	private $database= "test";

	/**
	 * Bandera para controlar el estado de la conexion
	 * @var boolean
	 */
	 
	private $connected = false;
	
	/**
	 * Guarda las consultas que se han hecho a la base de datos
	 * @var string
	 */
	
	private $query_history = '';
	
	/**
	 * Mantiene el recurso de conexion
	 * @var resource
	 */
	
	private $connection;
	
	//**********************************************************************************
	// Métodos mágicos
	//**********************************************************************************

	/**
	 * Constructor privado para evitar otras instancias
	 */

	private function __construct()
	{
   	}

	//**********************************************************************************
	// Métodos para el manejo de variables
	//**********************************************************************************
	
	/**
	 * Asigna el dominio de la base de datos
	 * @param string $host
	 */
	
	public function setHost( $host )
	{
		$this->host = $host;
	}

	/**
	 * Asigna el usuario de la base de datos
	 * @param string $user
	 */
	public function setUser( $user )
	{
		$this->user = $user;
	}

	/**
	 * Asigna la contraseña de la base de datos
	 * @param string $password
	 */
	public function setPassword( $password )
	{
		$this->password = $password;
	}

	/**
	 * Asigna el nombre de la base de datos
	 * @param string $database
	 */
	public function setDatabase( $database )
	{
		$this->database = $database;
	}
	
	//**********************************************************************************
	// Métodos para el procesamiento de acciones	
	//**********************************************************************************
	
	/**
	 * Ejecuta una sentencia SQL
	 * @param string $query
	 * @return resource
	 */

	public function query( $query )
	{
		if ( !$this->checkConnection() )
		{
			return NULL;
		}
		
		$result = mysql_query($query, $this->connection);
		if( !$result )
		{
			$this->query_history .= 'Error: '.$query."\n";
			trigger_error( 'Ha ocurrido un error al procesar los datos: '.$this->errorMessage()."\n", E_USER_WARNING );
			return NULL;	
		}
		else
		{
			$this->query_history .= $query."\n";
			return $result;
		}
	}
	
	/**
	 * Devuelve los datos de una fila dentro del resultado
	 * @param resource $result
	 * @param constant $type
	 * @return mixed
	 */
	
	public function fetch( $result, $type = self::F_OBJECT )
	{
		if( $type = self::F_OBJECT )
		{
			return mysql_fetch_object( $result );
		}
		elseif( $type = self::F_ROW )
		{
			return mysql_fetch_row( $result );	
		}
		elseif ( $type = self::F_ASSOC )
		{
			return mysql_fetch_assoc( $result );
		}
		else
		{
			trigger_error( 'Se ha enviado un parametro invalido a la función', E_USER_WARNING );
			return NULL;
		}
	}

	/**
	 * Obtiene un campo específico de una fila
	 * @param resource $result
	 * @param mixed $field
	 * @param int $row
	 * @return string
	 */
	
	public function result( $result, $field, $row = 0 )
	{
		return mysql_result( $result, $row, $field );
	}

	/**
	 * Devuelve el id autoincrementado de la ultima fila insertada
	 * @return int
	 */

	public function insertId()
	{
		return mysql_insert_id();
	}
	
	/**
	 * Devuelve el número de filas para el recurso
	 * @param resource $result
	 * @return int
	 */
	
	public function numRows( $result )
	{
		return mysql_num_rows( $result );
	}
	
	/**
	 * Evalúa los valores a ingresar en las sentencias SQL
	 * @param string $value
	 * @return string
	 */

	public function evalSQL( $value ) 
	{		
		if ( get_magic_quotes_gpc() )
		{
			stripslashes( $value );
		}
		
		if( $this -> checkConnection() )
		{
			$value = mysql_real_escape_string( $value, $this->connection );
		}
		
		return $value;
	}
	
	/**
	 * Devuelve el historial de consultas realizadas
	 * @return string
	 */
	
	public function showQueries()
	{
		return $this->query_history;
	}

	/**
	 * Verifica si la conexion esta activa
	 * @return boolean
	 */

	public function checkConnection()
	{
		if (!$this->connected)
		{
			return $this->defaultConnection();
		}
		elseif ( !mysql_ping( $this->connection ) )
		{
			mysql_close( $this->connection );
			$this->connected = false;
			return $this->defaultConnection();
		}
		else
		{
			return true;
		}
	}
	
	//**********************************************************************************
	// Métodos privados	
	//**********************************************************************************
	
	/**
	 * Conecta con la base de datos
	 * @return boolean
	 */
	
	private function defaultConnection()
	{
		$connection = @mysql_connect( $this->host, $this->user, $this->password );
		if (!$connection)
		{
			trigger_error( 'No se pudo establecer una conexión '. $this -> errorMessage(), E_USER_WARNING );
			return false;
		}

		$base = @mysql_select_db( $this->database, $connection);
		if(!$base)
		{
			trigger_error( 'No se pudo seleccionar la base de datos '. $this -> errorMessage(), E_USER_WARNING );
			return false;
		}
		
		$this->connected = true;
		$this->connection = $connection;
		
		return true;
	}

	/**
	 * Muestra el mensaje de error de mysql
	 * @return string
	 */

	private function errorMessage()
	{
		return mysql_error();
	}	
	
	//**********************************************************************************
	// Métodos estáticos	
	//**********************************************************************************
	
	/**
	 * Devuelve la instancia única de la clase
	 * @return Database
	 */

	public static function getInstance()
	{
		if( !self::$instance instanceof self )
		{
			self::$instance = new self;
		}

		return self::$instance;
	}
}

?>
