<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Vista base para salida en HTML
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 * @abstract
 */
 
abstract class HTMLView extends View
{
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************
	
	/**
	 * Página principal de la aplicación
	 * @var string
	 */
	protected $page;
	
	/**
	 * Plantillas utilizadas por la vista
	 * @var array
	 */
	protected $templates;
	
	/**
	 * Contenido principal de la página
	 * @var string
	 */
	protected $content;
	
	//**********************************************************************************
	// Métodos para el manejo de variables
	//**********************************************************************************
	
	/**
	 * Agrega contenido a la vista
	 * @param string $content
	 * @return void
	 */
	public function addContent( $content )
	{
		$this->content .= $content;
	}
	
	/**
	 * Modifica la página principal de la vista
	 * @param string $page
	 * @return void
	 */
	public function setPage( $page )
	{
		$this->page = $page;
	}
	
	//**********************************************************************************
	// Métodos para el procesamiento de acciones	
	//**********************************************************************************
	
	/**
	 * (non-PHPdoc)
	 * @see include/lib/core/View#show()
	 */
	public function show()
	{
		$this->display();
		
		$page = new Page( $this->page );
		$page->setVars( $this->getVars() );
		
		foreach( $this->templates as $name => $template )
		{
			$tpl = Template::load( $template, $this->vars );
			
			$page->addTemplate( $name, $template );
		}
		
		$buffer = Buffer::getInstance();
		$content =  $buffer->getContent( 'salida' ); 
		$page->setContent( $content.$this->content );
		
		return $page->show();
	}
		
	/**
	 * Agrega una plantilla a la vista
	 * @param string $name
	 * @param string $template
	 * @return void
	 */
	public function addTemplate( $name, $template )
	{
		$this->templates[$name] = $template;
	}
	
	//**********************************************************************************
	// Métodos privados	
	//**********************************************************************************
	
	/**
	 * Procesos especializados de la vista
	 * @return void
	 */
	abstract protected function display();
	
	/**
	 * (non-PHPdoc)
	 * @see include/lib/core/View#init()
	 */
	protected function init()
	{
		$this->page = Registry::get( 'DEFAULT_PAGE' );
		$this->templates = array();
	}	
}
 
?>
 