<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Prototipo de librería
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */
 
abstract class Lib
{
	//**********************************************************************************
	// Constantes
	//**********************************************************************************
	
	
	
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************
		
	/**
	 * Dependencias de la librería
	 * @var array
	 */
	protected $dependencies;
	
	//**********************************************************************************
	// Variables Públicas
	//**********************************************************************************
	
	/**
	 * Versión de MoonDragon compatible
	 * @var string
	 */
	public $moondragon_version;
	
	/**
	 * Número de versión de la librería
	 * @var string
	 */
	
	public $version;
		
	//**********************************************************************************
	// Métodos mágicos
	//**********************************************************************************
	
	/**
	 * Constructor de la clase
	 * 
	 * @final
	 * @return void
	 */
	final public function __construct()
	{
		$this->dependencies = array();
		
		$this->initVars();
	}
	
	//**********************************************************************************
	// Métodos para el manejo de variables
	//**********************************************************************************
	
	/**
	 * Inicializa los valores de la librería
	 * 
	 * @abstract
	 * @return void
	 */
	public abstract function initVars();
	
	//**********************************************************************************
	// Métodos para el procesamiento de acciones	
	//**********************************************************************************
	
	/**
	 * Ejecuta procesos automáticos de la librería
	 * 
	 * @return void
	 */
	public function autorun()
	{
		//No hace nada por defecto
	}
	
	//**********************************************************************************
	// Métodos privados	
	//**********************************************************************************
	
	/**
	 * Agrega una dependencia a la librería
	 * @param string $library
	 * @param string $version
	 * @return void
	 */
	private function addDependency( $library, $version )
	{
		$this->dependencies[$library] = $version;
	}
}
 
?>
 