<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Controla la publicación de una página web
 * 
 * @todo Para escribir esta clase es necesario que se defina el sistema de variables y los cargadores
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */
 
 class Page
{
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************
	
	/**
	 * Nombre de la página
	 * @var string
	 */
	private $page;
	
	/**
	 * Plantillas añadidas a la página
	 * @var array
	 */
	private $templates;
	
	/**
	 * Contenido principal de la página
	 * @var string
	 */
	private $content;
	
	/**
	 * Variables de contenido
	 * @var array
	 */
	private $vars = array();
	
	//**********************************************************************************
	// Métodos mágicos
	//**********************************************************************************
	
	/**
	 * Constructor de la clase
	 * @param string $page
	 * @return void
	 */
	public function __construct( $page )
	{
		$this->page = $page;
	}
	
	/**
	 * Método mágico para convertir la clase en una cadena
	 * @return string
	 */
	public function __toString()
	{
		return $this->show();
	}
	
	//**********************************************************************************
	// Métodos para el manejo de variables
	//**********************************************************************************

	/**
	 * Agrega una plantilla a la página
	 * @param string $name
	 * @param string $template
	 * @return void
	 */
	public function addTemplate( $name, $template )
	{
		$this->templates[$name] = $template;
	}
	
	/**
	 * Modifica el contenido principal de la página
	 * @param string $content
	 * @return void
	 */
	public function setContent( $content )
	{
		$this->content = $content;
	}
	
	/**
	 * Modifica las variables de contenido
	 * @param array $vars
	 * @return void
	 */
	public function setVars( $vars )
	{
		$this->vars = $vars;
	}
	
	//**********************************************************************************
	// Métodos para el procesamiento de acciones	
	//**********************************************************************************
	
	/**
	 * Muestra el contenido de la página
	 * @return string
	 */
	public function show()
	{
		$content = $this->process();
		
		return $content;
	}
	
	//**********************************************************************************
	// Métodos privados	
	//**********************************************************************************
	
	/**
	 * Procesa una página para agregar los fragmentos indicados
	 * @todo la adaptación de esta función se encuentra incompleta
	 *
	 * @return string
	 */

	private function process()
	{
		$template = new Template( 'pages/'.$this->page );
		$template->setVars( $this->vars );
		
		$htmlText = $template->show();
		$htmlText = str_replace ("'", "\'", $htmlText );
		$htmlText = preg_replace('/\[\@content\]/s', '\' . $this->getContent() . \'', $htmlText); //@todo Se puede implementar un sistema de etiquetas más elaborado
		//$htmlText = preg_replace('/\[\@css:(\w*)\]/s', '<link rel="stylesheet" type="text/css" href= "./css/\\1.css" />', $htmlText); //@todo Es necesario interactuar con el cargador de css
		$htmlText = preg_replace('/\[\@:(\w*)\]/s', '\' . $this->loadTemplate( \\1 ) . \'', $htmlText);
		eval("\$htmlText = '$htmlText';");
		$htmlText = str_replace ("\'", "'", $htmlText);

		return $htmlText;
	}
	
	/**
	 * Carga la plantilla seleccionada
	 * @param string $template
	 * @return string
	 */
	
	private function loadTemplate( $template )
	{
		if( isset( $this->templates[$template] ) )
		{
			return $this->templates[$template];
		}
		else
		{
			return Template::load( 'slices/'.$template, $this->vars );
		}
	}
	
	/**
	 * Devuelve el contenido principal
	 * @return string
	 */
	private function getContent()
	{
		return $this->content;
	}
}
 
 ?>
 