<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Almacena las secciones del sistema
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */
 
 class SectionTree
{
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************

	/**
	 * Secciones del sistema
	 * @var array
	 */
	private static $sections;

	
	//**********************************************************************************
	// Métodos estáticos	
	//**********************************************************************************

	/**
	 * Agrega una seccion al sistema
	 * @param string $id
	 * @param string $name
	 * @param string $level
	 * @param string $controller
	 * @param string $task
	 * @return void
	 */
	public static function addSection( $id, $name, $level, $controller = '', $task = '' )
	{
		if( $controller == '' )
		{
			$controller = Registry::get( 'DEFAULT_CONTROLLER' );
		}
		
		self::$sections[ $id ] = new Section( $id, $name, $level, $controller, $task );
	}
	
	/**
	 * Obtiene el controlador de la seccion
	 * @param string $section
	 * @return Controller
	 */
	
	public static function getControllerFrom( $id )
	{
		$section = self::$sections[ $id ];
		
		if( !($section instanceof Section) )
		{
			trigger_error( 'Se intento obtener el controlador de la seccion inexistente '.$id, E_USER_WARNING );
			
			$controller = Registry::get( 'DEFAULT_CONTROLLER' );
			return new $controller( 'unexistent' );
		}
		
		$controller = $section->get( 'controller' );
		
		if( !class_exists($controller) || !is_subclass_of( $controller, 'Controller' ) )
		{
			if( file_exists( 'include/controllers/class.'.strtolower($controller).'.php' ) )
			{
				include_once 'include/controllers/class.'.strtolower($controller).'.php';
			}
			
			if( !class_exists($controller) || !is_subclass_of( $controller, 'Controller' ) )
			{
				$controller = Registry::get( 'DEFAULT_CONTROLLER' );
			}
		}
		
		return new $controller( $section->get('task'), $id.'Model', $id.'View' );
	}
	
	/**
	 * Inicializa las variables de la seccion activa
	 * @param string $section
	 * @return void
	 */
	public static function initSection( $id )
	{
		$section = self::$sections[ $id ];
		
		if( $section instanceof Section )
		{
			Registry::set( 'section', $id );
			Registry::set( 'section_name', $section->get('name') );
			Registry::set( 'level', $section->get( 'level' ) );
		}
		else
		{
			trigger_error( 'Seccion '.$id.' no existe', E_USER_WARNING );// @todo localizar mensaje de error			
			Vars::set( 'Error_Message', 'La sección que ha intentado acceder no existe' ); // @todo localizar contenido
			
			Registry::set( 'section', Registry::get( 'ERROR_SECTION' ) );
			Registry::set( 'section_name', 'Sección no encontrada' );// @todo localizar mensaje
			Registry::set( 'level', Session::ANON );
		}
	}
}
 
 ?>
 