<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Clase para el manejo de sesiones
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */

class Session extends Singleton
{
	//**********************************************************************************
	// Constantes
	//**********************************************************************************
	
	/**
	 * Instancia única de la clase
	 * @var Session
	 */
	
	private static $instance;
	
	/**
	 * Identifica a los usuarios anónimos
	 * @var int
	 */
	const ANON = 0;
	
	/**
	 * Identifica a los usuarios con permisos estándar
	 * @var int
	 */
	const USER = 1;
	
	/**
	 * Identifica a los moderadores
	 * @var int
	 */
	const MOD = 2;
	
	/**
	 * Identifica a los administradores
	 * @var int
	 */
	const ADMIN = 3;
	
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************
		
	/**
	 * Nivel de autorización del usuario
	 * @var int
	 */
	private $auth_level;
	
	/**
	 * Identificador de la aplicación actual
	 * @var string
	 */
	private $app_id;
	
	/**
	 * Contraseña del usuario
	 * @var string
	 */
	private $user_password;
	
	//**********************************************************************************
	// Variables Públicas
	//**********************************************************************************
		
	/**
	 * Identificador del usuario
	 * @var int
	 */
	public $user_id;
	
	/**
	 * Nombre para mostrar del usuario
	 * @var string
	 */
	public $user_screenname;
	
	/**
	 * Correo electrónico del usuario
	 * @var string
	 */
	public $user_email;
	
	/**
	 * Identificador del grupo de usuario
	 * @var int
	 */
	public $group_id;
	
	//@todo Se pueden agregar más variables, como la versión del navegador y la ip, investigar la variable $_SERVER
	
	//**********************************************************************************
	// Métodos Mágicos
	//**********************************************************************************
	
	/**
	 * Constructor de la clase
	 * Privado para implementar el patrón Singleton
	 * Inicializa las variables de sessión e inicia la sesión
	 */
	private function __construct()
	{
		session_start();
		
		$id = $this->app_id = $_SERVER['PHP_SELF'];
		
		//@todo Inseguro, es recomendable conectarse a la base de datos
		if( isset( $_SESSION['auth_level_'.$id] ))
		{
			$this->auth_level = &$_SESSION['auth_level_'.$id];
		}
		else
		{
			$_SESSION['auth_level_'.$id] = Session::ANON;
			$this->auth_level = &$_SESSION['auth_level_'.$id];
		}
		
		
		if( isset( $_SESSION['user_id_'.$id] ))
		{
			$this->user_id = &$_SESSION['user_id_'.$id];
		}
		else
		{
			$_SESSION['user_id_'.$id] = 0;
			$this->user_id = &$_SESSION['user_id_'.$id];
		}
		
		if( isset( $_SESSION['user_password_'.$id] ))
		{
			$this->user_password = &$_SESSION['user_password_'.$id];
		}
		else
		{
			$_SESSION['user_password_'.$id] = '';
			$this->user_password = &$_SESSION['user_password_'.$id];
		}
		
		if( isset( $_SESSION['user_screenname_'.$id] ))
		{
			$this->user_screenname = &$_SESSION['user_screenname_'.$id];
		}
		else
		{
			$_SESSION['user_screenname_'.$id] = 'Invitado'; //@todo Será modificado al agregar soporte para lenguajes múltiples
			$this->user_screenname = &$_SESSION['user_screenname_'.$id];
		}

		if( isset( $_SESSION['user_email_'.$id] ))
		{
			$this->user_email = &$_SESSION['user_email_'.$id];
		}
		else
		{
			$_SESSION['user_email_'.$id] = '';
			$this->user_email = &$_SESSION['user_email_'.$id];
		}

		if( isset( $_SESSION['group_id_'.$id] ))
		{
			$this->group_id = &$_SESSION['group_id_'.$id];
		}
		else
		{
			$_SESSION['group_id_'.$id] = 0;
			$this->group_id = &$_SESSION['group_id_'.$id];
		}
   	}
   	
   	/**
   	 * Método mágico para accesar a propiedades del objeto
   	 * @param string $var
   	 * @return mixed
   	 */
   	public function __get( $var )
   	{
   		if( isset($_SESSION[$var.'_'.$this->app_id] ))
   		{
   			return $_SESSION[$var.'_'.$this->app_id];
   		}
   		else
   		{
   			return NULL;
   		}	
   	}
   	
   	/**
   	 * Método mágico para modificar las propiedad del objeto
   	 * @param string $var
   	 * @param mixed $value
   	 * @return void
   	 */
   	
   	public function __set( $var , $value )
   	{
   		$_SESSION[$var.'_'.$this->app_id] = $value;
   	}
   	
   	/**
   	 * Método mágico para comprobar si una propiedad existe
   	 * @param string $var
   	 * @return boolean
   	 */
   	public function __isset( $var )
   	{
   		return isset($_SESSION[$var.'_'.$this->app_id]);
   	}
   	
   	/**
   	 * Método mágico para destruir una propiedad del objeto
   	 * @param string $var
   	 * @return void
   	 */
	public function __unset( $var )
   	{
   		unset($_SESSION[$var.'_'.$this->app_id]);
   	}
   	
   	//**********************************************************************************
	// Métodos para el manejo de variables
	//**********************************************************************************
	
   	/**
   	 * Método para accesar a las propiedades del objeto
   	 * @param string $var
   	 * @return mixed
   	 */
	public function get( $var )
   	{
   		if( isset($_SESSION[$var.'_'.$this->app_id] ))
   		{
   			return $_SESSION[$var.'_'.$this->app_id];
   		}
   		else
   		{
   			return NULL;
   		}   		
   	}
   	
   	/**
   	 * Método para modificar las propiedades del objeto
   	 * @param string $var
   	 * @param mixed $value
   	 * @return void
   	 */
   	public function set( $var , $value )
   	{
   		if( $var != 'auth_level' && $var != 'user_password' && $var != 'app_id' && $var != 'instance' )
   		{
   			$_SESSION[$var.'_'.$this->app_id] = $value;
   		}
   		else
   		{
   			trigger_error( "Se intento modificar una variable privada\n", E_USER_WARNING );
   		}
   	}
   	
   	//**********************************************************************************
	// Métodos para el procesamiento de acciones	
	//**********************************************************************************
	
   	/**
   	 * Almacena la información estándar de la sesión
   	 * @param int $user_id
   	 * @param string $user_password
   	 * @param string $user_screename
   	 * @param string $user_email
   	 * @param int $group_id
   	 * @param int $auth_level puede ser Session::ANON, Session::USER, Session::MOD o Session::ADMIN
   	 * @return boolean true si la información es correcta
   	 */
   	public function sessionCreate( $user_id, $user_password, $user_screename, $user_email, $group_id = 0, $auth_level = Session::ANON )
   	{
   		$this->user_id = $user_id;
   		$this->user_password = $user_password;
   		$this->user_screenname = $user_screename;
   		$this->user_email = $user_email;
   		$this->group_id = $group_id;
   		
   		$this->auth_level = $auth_level;//@todo Posible error de seguridad, confrontar contra bdd
   		
   		return true;
   	}
   	
   	/**
   	 * Borra toda la información estándar de la sesión actual
   	 * @return void
   	 */   	
   	public function sessionDestroy()
   	{
   		$this->user_id = 0;
   		$this->user_password = '';
   		$this->user_screenname = 'Invitado'; //@todo Será cambiado al agregar soporte a varios idiomas
   		$this->user_email = '';
   		$this->group_id = 0;
   		
   		$this->auth_level = Session::ANON;
   	}
   	
   	/**
   	 * Autentica el nivel de la sesión
   	 * @param int $level puede ser Session::ANON, Session::USER, Session::MOD o Session::ADMIN
   	 * @return boolean true si la sesión actual tiene los permisos necesarios
   	 */
   	public function authenticate( $level )
   	{
   		if( $this->auth_level >= $level )
   		{
   			return true;
   		}
   		else
   		{
   			return false;
   		}
   	}
   	
	//**********************************************************************************
	// Métodos estáticos	
	//**********************************************************************************
	
	/**
	 * Devuelve la instancia única de la clase
	 * @return Session
	 */

	public static function getInstance()
	{
		if( !self::$instance instanceof self )
		{
			self::$instance = new self;
		}

		return self::$instance;
	}
}

?>
