<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Clase base para utilizar el patrón Singleton
 * 
 * @abstract
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */
 
abstract class Singleton
{
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************
	
	/**
	 * Instancia única de la clase
	 * @var Singleton
	 */
	
	private static $instance;
		
	//**********************************************************************************
	// Métodos mágicos
	//**********************************************************************************
	
	/**
     * Invalida la clonación de la clase
     * @return void
     */
    
   	final public function __clone()
   	{
     	trigger_error("Operación Invalida: No puedes clonar una instancia de ". get_class($this) .".", E_USER_ERROR );
   	}
  
  	/**
  	 * Invalida la deserialización de la clase
  	 * @return void
  	 */
  	
   	final public function __wakeup()
   	{
      	trigger_error("No puedes deserializar una instancia de ". get_class($this) .".", E_USER_ERROR );
   	}
	
	//**********************************************************************************
	// Métodos estáticos	
	//**********************************************************************************
	
	/**
	 * Devuelve la instancia única de la clase
	 * @abstract
	 * @return Singleton
	 */

	public abstract static function getInstance();
		
	/*
	 * Implementacion de la clase
	 * Debe aplicarse en la clase hija para que instancie el objeto correcto
	 */
	 
	/*
	 
	public static function getInstance()
	{
		if( !self::$instance instanceof self )
		{
			self::$instance = new self;
		}

		return self::$instance;
	}
	
	*/
}
 
 ?>
 