<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Manejador de plantillas
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */
 
 class Template
{
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************
	
	/**
	 * Ruta del directorio de plantillas
	 * @var string
	 */
	private $path = './templates/';
	
	/**
	 * Extensión de archivo para las plantillas
	 * @var string
	 */
	private $ext = '.tpl';
	
	/**
	 * Nombre del directorio actual de plantillas
	 * @var string
	 */
	private $dir = 'common';
	
	/**
	 * Nombre de la plantilla actual
	 * @var string
	 */
	private $template;
			
	/**
	 * Variables para ser reemplazadas en la plantilla
	 * @var array
	 */
	private $vars = array();
	
	/**
	 * Contenido de la plantilla
	 * @var string
	 */
	private $content = '';
	
	//**********************************************************************************
	// Métodos mágicos
	//**********************************************************************************
	
	/**
	 * Constructor de la clase
	 * @param string $file
	 * @return void
	 */
	public function __construct( $template = '' )
	{
		$this->setTemplate( $template );
	}
	
	/**
	 * Método mágico para convertir el objeto en una cadena
	 * @return string
	 */
	public function __toString()
	{
		return $this->show();
	}
	
	//**********************************************************************************
	// Métodos para el manejo de variables
	//**********************************************************************************
	
	/**
	 * Modifica el directorio usado por la plantilla
	 * @param string $dir
	 * @return void
	 */
	public function setTemplateDir( $dir )
	{
		$this->dir = $dir;
	}
	
	/**
	 * Modifica la plantilla
	 * @param string $template
	 * @return void
	 */
	
	public function setTemplate( $template )
	{
		if( strpos( $template, '/' ) !== false )
		{
			list( $dir, $template ) = explode( '/', $template );
			$this->dir = $dir;
		}
		$this->template =  $template;
		$this->content = '';
	}
	
	/**
	 * Modifica las variables que serán agregadas a la plantilla
	 * @param array $vars
	 * @return void
	 */
	public function setVars( $vars )
	{
		$this->vars = $vars;//@todo agregar compatibilidad con variables globales
	}
	
	/**
	 * Modifica la ruta donde se encuentran las plantillas
	 * @param string $path
	 * @return void
	 */
	public function setPath( $path )
	{
		$this->path = $path;
	}
	
	/**
	 * Modifica la extensión de archivo para las plantillas
	 * @param string $ext
	 * @return void
	 */
	public function setExtension( $ext )
	{
		$this->ext = $ext;
	}
	
	//**********************************************************************************
	// Métodos para el procesamiento de acciones	
	//**********************************************************************************
	
	/**
	 * Devuelve el contenido de la plantilla despues de procesarlo
	 * @return string
	 */
	public function show()
	{
		if( $this->content == '' )
		{
			if( $this->readTemplate() )
			{
				$this->replaceVars();
			}
		}
		
		return $this->content;
	}
	
	//**********************************************************************************
	// Métodos privados	
	//**********************************************************************************
		
	/**
	 * Lee el archivo de plantilla
	 * @return boolean
	 */
	private function readTemplate()
	{
		$file =  $this->path. $this->dir.'/'. $this->template. $this->ext;
		
		if( !file_exists( $file ) )
		{
			trigger_error( 'La plantilla '.$file.' no existe', E_USER_WARNING );//@todo agregar localización
			return false;
		}
		
		$file_data = @fopen($file, 'r');
		if ( !$file_data ) 
		{
			$this->content = '';
			trigger_error( 'No se pudo leer la plantilla' , E_USER_WARNING );//@todo agregar localización
			return false;
		}
		
		$this->content = fread( $file_data, filesize( $file ) );
		fclose( $file_data );
		
		return true;
	}
	
	/**
	 * Reemplaza las variables en la plantilla
	 * @return void
	 */
	private function replaceVars()
	{
		if ( !empty($this->vars) ) 
		{
			$content = str_replace ( "'", "\'", $this->content );
			
			$content = preg_replace( '/\[\#:(\w*)\]/s', "' . $\\1 . '", $content );
			reset ($this->vars);
			while (list($key, $val) = each($this->vars)) 
			{
					$$key = $val;
			}
			eval("\$content = '$content';");
			reset ($this->vars);
			while (list($key, $val) = each($this->vars)) 
			{
					unset($$key);
			}
			$this->content = str_replace ("\'", "'", $content);
		}
		else
		{
			trigger_error( 'No se han definido variables para la plantilla', E_USER_NOTICE );//@todo agregar localización
			$this->content = preg_replace( '/\[\#:(\w*)\]/s', '', $this->content );			
		}
	}
	
	//**********************************************************************************
	// Métodos estáticos	
	//**********************************************************************************
	
	/**
	 * Procesa y devuelve la plantilla específicada
	 * @param string $template
	 * @param array[optional] $vars
	 * @return string
	 */
	
	public static function load( $template, $vars = array() )
	{
		$tpl = new self( $template );
		$tpl->setVars( $vars );
		return $tpl->show();
	}
}
 
 ?>
 