<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Vista base para MoonDragon
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 * @abstract
 */
 
abstract class View
{
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************
	
	/**
	 * Variables que serán mostradas en la vista
	 * @var array
	 */
	protected $vars;
	
	//**********************************************************************************
	// Métodos mágicos
	//**********************************************************************************
	
	/**
	 * Constructor base de la vista
	 * @return void
	 * @final
	 */
	final public function __construct()
	{
		$this->init();
		$this->vars = array();
		Registry::set( 'view', $this );	
	}
	
	/**
	 * Método  mágico que convierte el objeto en una cadena
	 * @return string
	 */
	public function __toString()
	{
		return $this->show();
	}
	
	//**********************************************************************************
	// Métodos para el manejo de variables
	//**********************************************************************************
	
	/**
	 * Agrega una variable a la vista
	 * @param string $name
	 * @param string $value
	 * @return void
	 */
	public function addVar( $name, $value )
	{
		$this->vars[$name] = $value;
	}
	
	/**
	 * Devuelve el arreglo de variables que se usarán en la vista
	 * @return array
	 */	
	public function getVars()
	{
		$vars = array_merge(  Vars::getVars(), $this->vars );
		
		$buffer = Buffer::getInstance();
				
		$vars['system_warning'] = $buffer->getContent( 'e_warning', false );
		$vars['system_notice'] = $buffer->getContent( 'e_notice', false ); 
		
		return $vars;
	}
	
	//**********************************************************************************
	// Métodos para el procesamiento de acciones	
	//**********************************************************************************
	
	/**
	 * Muestra el contenido de la vista
	 * @abstract
	 * @return string
	 */
	abstract public function show();
	
	//**********************************************************************************
	// Métodos privados	
	//**********************************************************************************
	
	/**
	 * Inicializa las propiedades del objeto
	 * @return void
	 */
	protected function init()
	{
	}

}
 
?>
 